from pl_user_io.str_input import str_input


def int_input(question: str) -> int:
    response = str_input(question)
    try:
        return int(response)
    except ValueError:
        return int_input(question)



import re

import pytest

from pl_user_io.testing.user_io_fake import user_io_fake

type Question = str
type Answer = str


def all_instances_of_question(question: Question) -> list[str]:
    number_select_regex = r"\(1-\d+\)"
    minimum_any_characters_regex = r".+?"
    return re.findall(
        rf"{re.escape(question)}\n{minimum_any_characters_regex}\n{number_select_regex}",
        user_io_fake().output_str(),
        re.DOTALL,
    )


def options_for_multiple_choice(question: str, index: int = 0) -> list[str]:
    result = all_instances_of_question(question)
    if len(result) == 0:
        pytest.fail(f"Multiple choice question not found: `{question}`")
    return [line.split(" ", maxsplit=1)[1] for line in result[index].split("\n")[1:-1]]



from pl_user_io.display import display
from pl_user_io.int_input import int_input


def multiple_choice[T](question: str, options: list[tuple[str, T]]) -> T:
    display(f"{question}")
    for i, option in enumerate(options):
        display(f"{i + 1}. {option[0]}")
    try:
        choice = int_input(f"(1-{len(options)})")
        if choice < 1 or choice > len(options):
            raise IndexError
        result = options[choice - 1][1]
    except IndexError:
        display()
        return multiple_choice(question, options)
    else:
        display()
        return result



import re
from collections.abc import Callable
from unittest.mock import Mock

from pl_user_io.str_input import StrInputResponse
from pl_user_io.testing.helpers import Answer, Question, all_instances_of_question
from pl_user_io.testing.user_io_fake import InputMappingCallback, user_io_fake

StrInputMock = Mock


def stub_str_input_callback(callback: InputMappingCallback) -> None:
    user_io_fake().input_mapping_callbacks.insert(0, callback)


def stub_str_input(response: StrInputResponse, prior_output: str) -> None:
    def _callback(output: str) -> StrInputResponse | None:
        if output.endswith(prior_output):
            return response
        return None

    user_io_fake().input_mapping_callbacks.insert(0, _callback)


def input_count(mock: StrInputMock) -> int:
    return mock.call_count


def stub_yes_no(question: str, answer: bool) -> None:
    stub_str_input("y" if answer else "n", f"{question} (y/n)")


def stub_int_input(question: str, answer: int) -> None:
    stub_str_input(str(answer), f"{question}")


type IntInputMappingCallback = Callable[[str], int | None]


def stub_int_input_callback(callback: IntInputMappingCallback) -> None:
    stub_str_input_callback(
        lambda output: str(callback(output)) if callback(output) is not None else None,
    )


def _extract_answer_number(multiple_choice_output: str, answer: Answer) -> int:
    regex = rf"(\d+)\. {re.escape(answer)}"
    match = re.search(regex, multiple_choice_output, re.DOTALL)
    if match is None:
        msg = f"Failed to find answer `{answer}` in output: ```\n {multiple_choice_output}\n```"
        raise Exception(msg)
    return int(match.group(1))


def stub_multiple_choice(question: Question, answer: Answer) -> None:
    def _callback(output: str) -> int | None:
        result = all_instances_of_question(question)
        if len(result) != 0 and output.endswith(result[-1]):
            return _extract_answer_number(result[-1], answer)
        return None

    stub_int_input_callback(_callback)
